<?php
/**
 * Copyright (c) 2019, Nosto Solutions Ltd
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @author Nosto Solutions Ltd <contact@nosto.com>
 * @copyright 2019 Nosto Solutions Ltd
 * @license http://opensource.org/licenses/BSD-3-Clause BSD 3-Clause
 *
 */

namespace Nosto\Tagging\Model\Service\Product\Category;

use Magento\Catalog\Model\Category;
use Magento\Catalog\Model\Product;
use Magento\Store\Api\Data\StoreInterface;
use Nosto\Tagging\Logger\Logger;

class CachingCategoryService implements CategoryServiceInterface
{

    /** @var Logger */
    private $nostoLogger;

    /**
     * @var CategoryServiceInterface
     */
    private $categoryService;

    /**
     * @var array
     */
    private $cache = [];

    /**
     * Index constructor.
     * @param Logger $nostoLogger
     * @param CategoryServiceInterface $categoryService
     */
    public function __construct(
        Logger $nostoLogger,
        CategoryServiceInterface $categoryService
    ) {
        $this->nostoLogger = $nostoLogger;
        $this->categoryService = $categoryService;
    }

    /**
     * @inheritDoc
     */
    public function getCategory(Category $category, StoreInterface $store)
    {
        if (!isset($this->cache[$store->getId()])) {
            $this->cache[$store->getId()] = [];
        }

        if (!isset($this->cache[$store->getId()][$category->getId()])) {
            $slug = $this->categoryService->getCategory($category, $store);
            $this->cache[$store->getId()][$category->getId()] = $slug;
        }
        return $this->cache[$store->getId()][$category->getId()];
    }

    /**
     * @inheritDoc
     */
    public function getCategories(Product $product, StoreInterface $store)
    {
        return $this->categoryService->getCategories($product, $store);
    }
}
